//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#ifndef GLBOY_CORE_HH
#define GLBOY_CORE_HH

#include "glBoy.hh"
#include "DWORD.hh"
#include "MemoryMap.hh"
#include "Registers.hh"

#include <deque>
#include <tr1/functional>
#include <functional>
#include <vector>

namespace glBoy
{

class Core
{
public:
	Core();

	MemoryMap& getMemoryMap() { return m_mem; }

	Registers& getRegisters() { return m_reg; }

	void run();

	cycle_t getClockCount() const { return m_clock; }

	bit getIFF2() const { return m_iff2; }

	void raiseInterrupt(int interrupt);
	void halt() { m_halted = true; }

	typedef int CallbackId;
	typedef std::function<void (CallbackId)> PeriodicCallbackFn;

	CallbackId registerPeriodic(
		int clockPeriod, int offset, const PeriodicCallbackFn& callback
		);

private:
	uint8_t op8();
	DWORD op16();

	// The Z80 can switch between register sets.
	Registers m_reg;
	Registers m_regDash;

	// Interrupt enablement
	bit m_iff1;
	bit m_iff2;

	enum InterruptMode { IM_0, IM_1, IM_2 };
	InterruptMode m_interruptMode;

	bool m_halted;
	std::deque<uint8_t> m_interruptQueue;

	MemoryMap m_mem;

	cycle_t m_clock;

	struct PeriodicCallback
	{
		int clocks;
		cycle_t nextPeriod;
		PeriodicCallbackFn callback;
	};
	std::vector<PeriodicCallback> m_periodicCallbacks;
	cycle_t m_nextPeriodicCallback;
};

} // namespace glBoy

#endif
