//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#ifndef GLBOY_DWORD_HH
#define GLBOY_DWORD_HH

#include "glBoy.hh"
#include "util.hh"

#include <iostream>

namespace glBoy
{

struct __attribute__ ((__packed__)) DWORD
{
	// Always constructed, and stored, in Z80 byte order.
	union
	{
		uint16_t dword;
		struct
		{
			reg8_t l;
			reg8_t h;
		};
	};

	uint16_t z80() const { return dword; }
	uint16_t host() const
	{
#ifdef HOST_LITTLE_ENDIAN
		return dword;
#else
		return (uint16_t(h) << 8) | l;
#endif
	}

	void inc(uint8_t val = 1)
	{
#ifdef HOST_LITTLE_ENDIAN
		dword += val;
#else
		uint16_t tmp(host() + val);
		l = tmp & 0xf;
		h = tmp >> 8;
#endif
	}

	void dec(uint8_t val = 1)
	{
#ifdef HOST_LITTLE_ENDIAN
		dword -= val;
#else
		uint16_t tmp(host() - val);
		l = tmp & 0xf;
		h = tmp >> 8;
#endif
	}
};

inline std::ostream& operator<<(std::ostream& out, const glBoy::DWORD& val)
{
	out << glBoy::hex(val.host());
	return out;
}

// Create a dword from z80-ordered mem.
inline DWORD CreateDWORD(uint8_t* in)
{
	DWORD result;
	result.l = in[0];
	result.h = in[1];
	return result;
}

#if defined(HOST_LITTLE_ENDIAN)
inline DWORD htoz(uint16_t host)
{
	DWORD result;
	result.dword = host;
	return result;
}
#else
inline DWORD htoz(uint16_t host)
{
	DWORD result;
	result.dword = (host << 8) | (host >> 8);
	return result;
}
#endif
} // namespace glBoy

#endif


