//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#include "glBoy.hh"
#include "GameboyJoypad.hh"
#include "Core.hh"

#include <algorithm>
#include <cassert>
#include <cstdlib>

using namespace glBoy;

// TODO implement interrupt! 


GameboyJoypad::GameboyJoypad(Core& core) :
	m_state(Joypad_Off)
{
	m_inputs[DOWN].sdlKey = SDLK_DOWN;
	m_inputs[UP].sdlKey = SDLK_UP;
	m_inputs[LEFT].sdlKey = SDLK_LEFT;
	m_inputs[RIGHT].sdlKey = SDLK_RIGHT;
	m_inputs[START].sdlKey = SDLK_RETURN;
	m_inputs[SELECT].sdlKey = SDLK_BACKSPACE;
	m_inputs[A].sdlKey = SDLK_a;
	m_inputs[B].sdlKey = SDLK_s;

	pollInput(0);

	using namespace std::placeholders;
	core.registerPeriodic(
		32768, // 128Hz
		0,
		std::bind(&glBoy::GameboyJoypad::pollInput, this, _1)
		);
}

uint8_t
GameboyJoypad::read8(uint16_t)
{
	uint8_t result(0xff);

	switch (m_state)
	{
	case Joypad_Off:
		result = 0xff;
		break;

	case Joypad_Directional:
		result = m_directionalState;
		break;

	case Joypad_Button:
		result = m_buttonState;
		break;
	};
	return result;
}


void
GameboyJoypad::write8(uint16_t, uint8_t value)
{
	switch (value)
	{
	case 0x20:
		m_state = Joypad_Directional;
		break;
	case 0x10:
		m_state = Joypad_Button;
		break;
	case 0x30:
		m_state = Joypad_Off;
		break;
	default:
		assert(false);
	}
}

void
GameboyJoypad::pollInput(Core::CallbackId)
{
	SDL_Event event;

	while (SDL_PollEvent(&event))
	{
		switch(event.type)
		{
			case SDL_KEYUP:
			case SDL_KEYDOWN:
			{
				for (int i = 0; i < 8; ++i)
				{
					if (event.key.keysym.sym == m_inputs[i].sdlKey)
					{
						m_inputs[i].depressed = (event.key.type == SDL_KEYDOWN);
						break;
					}
				}
			}; break;

			case SDL_QUIT:
				exit(0);
				break;

			default:
				break;
		}
	}

	m_directionalState = 0xff;
	if (m_inputs[DOWN].depressed) m_directionalState ^= 0x8;
	if (m_inputs[UP].depressed) m_directionalState ^= 0x4;
	if (m_inputs[LEFT].depressed) m_directionalState ^= 0x2;
	if (m_inputs[RIGHT].depressed) m_directionalState ^= 0x1;

	m_buttonState = 0xff;
	if (m_inputs[START].depressed) m_buttonState ^= 0x8;
	if (m_inputs[SELECT].depressed) m_buttonState ^= 0x4;
	if (m_inputs[B].depressed) m_buttonState ^= 0x2;
	if (m_inputs[A].depressed) m_buttonState ^= 0x1;
}

