//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#ifndef GLBOY_GAMEBOYTIMER_HH
#define GLBOY_GAMEBOYTIMER_HH

#include "glBoy.hh"
#include "Core.hh"

#include "MemoryMap.hh"

#include <time.h>

#include <SDL/SDL.h>

namespace glBoy
{

class GameboyTimer : public MemoryMap::Memory
{
public:
	GameboyTimer(
		Core& core,
		const std::function<void(void)>& setFrameSkip,
		const std::function<void(void)>& clearFrameSkip
		);

	void tick(Core::CallbackId id);

	virtual uint8_t read8(uint16_t address);
	virtual void write8(uint16_t address, uint8_t value);

private:
	enum Reg { DIV = 0, TIMA, TMA, TAC };

	Core& m_core;
	std::function<void(void)> m_setSkipFrame;
	std::function<void(void)> m_clearSkipFrame;

	uint8_t m_reg[4];

	timespec m_lastSlept;

	int m_clockSleepDelay;
	int m_clockSleepCounter;
};

} // namespace glBoy

#endif
