//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#include "Log.hh"

#include <cstdlib>
#include <iostream>
#include <sstream>

using namespace glBoy;

Log&
Log::Instance()
{
	static Log s_instance;
	return s_instance;
}

Log::Log() :
	m_levelMask(7),
	m_nullStream(&m_nullBuf)
{
	char* env(getenv("GLBOY_DEBUG_LEVEL"));
	if (env)
	{
		std::stringstream stream;
		stream << env;

		// If this conversion fails, level mask remains 0, which is ok.
		stream >> m_levelMask;
	}
}

void
Log::setDebugLevel(uint32_t levelMask)
{
	m_levelMask = levelMask;
}

bool
Log::isDebug(DebugLevel level) const
{
	return (m_levelMask & level) != 0;
}

std::ostream&
Log::operator()(DebugLevel level)
{
	if (isDebug(level))
	{
		return std::cout;
	}
	else
	{
		return m_nullStream;
	}
}

