//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#ifndef GLBOY_LOG_HH
#define GLBOY_LOG_HH

#include "glBoy.hh"

#include <cstdint>
#include <ostream>
#include <streambuf>

namespace glBoy
{
	class Log
	{
	public:

		enum DebugLevel 
		{
			DBG_STAT = 1, DBG_ERROR = 2, DBG_WARNING = 4, DBG_MEMORY = 8, DBG_TRACE = 16
		};
		static Log& Instance();

		void setDebugLevel(uint32_t levelMask);

		bool isDebug(DebugLevel level) const;
		std::ostream& operator()(DebugLevel level);

	private:
		Log();

		Log(const Log&);
		Log& operator=(const Log&);

		uint32_t m_levelMask;

		class NullBuf : public std::streambuf
		{
		public:
		};

		NullBuf m_nullBuf;
		std::ostream m_nullStream;
	};

} // namespace glBoy
#endif
