//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#include "glBoy.hh"
#include "Registers.hh"

#include "Core.hh"

#include <string.h>

using namespace glBoy;

void
Flags::set(op8_t operand, const Core& context)
{
	S = static_cast<int8_t>(operand) < 0 ? 1 : 0;
	Z = operand == 0 ? 1 : 0;
	U5 = (operand & 0b00010000) >> 4;
	H = 0;
	U3 = (operand & 0b00000100) >> 2;
	P = context.getIFF2();
	N = 0;
	// C is not affected
}

void
Flags::setCounter(DWORD BC)
{
	// S is not affected
	// Z is not affected
	U5 = (BC.host() & 0b00010000) >> 4;
	H = 0;
	U3 = (BC.host() & 0b00000100) >> 2;
	P = BC.host() ? 1 : 0;
	N = 0;
	// C is not affected
}

void
Flags::setSub(op8_t A, op8_t B, DWORD BC)
{
	S = (A > B) ? 1 : 0;
	Z = (A == B) ? 1 : 0;
	U5 = (A & 0b00010000) >> 4;
	H = (B & 0xF) > (A & 0xF);
	U3 = (A & 0b00000100) >> 2;
	P = BC.host() ? 1 : 0;
	N = 1;
	// C is not affected
}

void
Registers::reset()
{
	memset(this, 0, sizeof(Registers));
}

