// hq4x OpenGL Shader
//----------------------------------------------------------
// Copyright (C) 2003 MaxSt ( maxst@hiend3d.com )
// Copyright 2011 Michael McMaster <michael@codesrc.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public
//License as published by the Free Software Foundation; either
//version 3 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU General Public
//License along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//----------------------------------------------------------
//
// Changes from original code:
//  - Converted to GLSL, inspired by the XNA port of hqNx by
// Phill Djonov <phill@vec3.net> (http://github.com/pdjonov/hqnx/wiki)
// - Applied clause 3 of LGPL 2.1 to alter license to GNU GPL 3.  Updated
// License notice at top of file as required by clause 3.

#define GL_GLEXT_PROTOTYPES

#include "hq4x.hh"

#include <GL/glu.h>
#include <GL/glext.h>

#include <cassert>
#include <cstdint>
#include <cmath>
#include <cstdlib>
#include <iomanip>
#include <iostream>

using namespace glBoy;

namespace
{

const char*
s_VectorSource =
"\
//#version 130\n\
\
uniform vec4 mmTextureSize;\n\
\
varying vec4 c5;\n\
\
void main()\n\
{\n\
	c5 = gl_MultiTexCoord0;\n\
\
	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;\n\
}\n\
";

const char*
s_FragmentSource =
"\
// Enable bitwise operators\n\
#version 130\n\
\
uniform sampler2D mmTexture;\n\
uniform sampler2D mmLookup;\n\
uniform vec4 mmTextureSize;\n\
\
in vec4 c5;\n\
\
out vec4 fragColour;\n\
\
vec4 RGB2YUV(in vec4 val)\n\
{\n\
	return mat4(\
		0.299, 0.587, 0.114, 0,\n\
		-0.14713, -0.28886, 0.436, 0,\n\
		0.615, -0.51499, -0.10001, 0,\n\
		0, 0, 0, 0\n\
		) * val;\
}\n\
\
bool Diff(in vec4 YUV1, in vec4 YUV2)\n\
{\n\
	return any(\n\
		greaterThan(\n\
			abs(YUV1 - YUV2),\n\
			vec4(48.0/255.0, 7.0/255.0, 6.0/255.0, 0)\n\
			).xyz\n\
		);\n\
}\n\
\
void main()\n\
{\n\
	vec4 c_offsetX = vec4(1.0/(mmTextureSize.x-1), 0, 0, 0);\n\
	vec4 c_offsetY = vec4(0, 1.0/(mmTextureSize.y-1), 0, 0);\n\
\
	vec4 w[9];\n\
	w[0] = texture2DProj(mmTexture, c5 - c_offsetY - c_offsetX);\n\
	w[1] = texture2DProj(mmTexture, c5 - c_offsetY);\n\
	w[2] = texture2DProj(mmTexture, c5 - c_offsetY + c_offsetX);\n\
\
	w[3] = texture2DProj(mmTexture, c5 - c_offsetX);\n\
	w[4] = texture2DProj(mmTexture, c5);\n\
	w[5] = texture2DProj(mmTexture, c5 + c_offsetX);\n\
\
	w[6] = texture2DProj(mmTexture, c5 + c_offsetY - c_offsetX);\n\
	w[7] = texture2DProj(mmTexture, c5 + c_offsetY);\n\
	w[8] = texture2DProj(mmTexture, c5 + c_offsetY + c_offsetX);\n\
\
	vec4 YUV[9];\n\
	YUV[0] = RGB2YUV(w[0]);\n\
	YUV[1] = RGB2YUV(w[1]);\n\
	YUV[2] = RGB2YUV(w[2]);\n\
	YUV[3] = RGB2YUV(w[3]);\n\
	YUV[4] = RGB2YUV(w[4]);\n\
	YUV[5] = RGB2YUV(w[5]);\n\
	YUV[6] = RGB2YUV(w[6]);\n\
	YUV[7] = RGB2YUV(w[7]);\n\
	YUV[8] = RGB2YUV(w[8]);\n\
\
	int pattern = 0;\n\
	int flag = 1;\n\
	int k;\n\
	for (k = 0; k < 9; k++)\n\
	{\n\
		if (k == 4) continue;\n\
\
		if (Diff(YUV[4], YUV[k]))\n\
		{\n\
			pattern = pattern + flag;\n\
		}\n\
		flag = flag * 2;\n\
	}\n\
\
	// k is now diff.\n\
	k = 0;\n\
	if (Diff(YUV[1], YUV[5]))\n\
	{\n\
		k = k + 1;\n\
	}\n\
	if (Diff(YUV[5], YUV[7]))\n\
	{\n\
		k = k + 2;\n\
	}\n\
	if (Diff(YUV[7], YUV[3]))\n\
	{\n\
		k = k + 4;\n\
	}\n\
	if (Diff(YUV[3], YUV[1]))\n\
	{\n\
		k = k + 8;\n\
	}\n\
\
	vec2 lookupMax = vec2(1.0/47.0, 1.0/4095);\n\
	ivec2 thisPixel = ivec2(fract(c5 * mmTextureSize) * 4.0);\n\
	ivec2 lookupIndex = ivec2((thisPixel.y*4+thisPixel.x)*3, (pattern * 16) + k);\n\
	vec4 weight1 = texture2D(mmLookup, lookupIndex * lookupMax);\n\
	vec4 weight2 = texture2D(mmLookup, (lookupIndex + ivec2(1, 0)) * lookupMax);\n\
	vec4 weight3 = texture2D(mmLookup, (lookupIndex + vec2(2, 0)) * lookupMax);\n\
\
	// Dodgy fix for inaccurate lookups :-( Seemed to work on Quadro NVS 290 without this\n\
	float allW = dot(\n\
		vec4(1,1,1,0),\n\
		vec4(\n\
			dot(weight1, vec4(1,1,1,0)),\n\
			dot(weight2, vec4(1,1,1,0)),\n\
			dot(weight3, vec4(1,1,1,0)),\n\
			0\n\
			)\n\
		);\n\
	fragColour =\n\
		w[0]*weight1.x + w[1]*weight1.y + w[2]*weight1.z +\n\
		w[3]*weight2.x + w[4]*weight2.y + w[5]*weight2.z +\n\
		w[6]*weight3.x + w[7]*weight3.y + w[8]*weight3.z;\n\
	fragColour = fragColour / allW;\n\
\
//if (allW > 1.100) { fragColour =  vec4(0.0,0.0,1.0,0); } \n\
//if (allW < 0.9) { fragColour =  vec4(0.0,1.0,0.0,0); } \n\
\
}\n\
";

void abortGLError()
{
	std::cerr << "Fatal OpenGL error in hq4x shader: " <<
		gluErrorString(glGetError()) << std::endl;
	abort();
}

void abortShaderError(GLint shader)
{
	std::cerr << "Fatal error using hq4x shader: " <<
		gluErrorString(glGetError()) << std::endl;

	char buffer[2048];
	int len;
	glGetShaderInfoLog(shader, 2048, &len, &buffer[0]); 
	std::cerr << buffer << std::endl;

	abort();
}

void abortProgramError(GLint program)
{
	std::cerr << "Fatal error using hq4x program: " <<
		gluErrorString(glGetError()) << std::endl;

	char buffer[2048];
	int len;
	glGetProgramInfoLog(program, 2048, &len, &buffer[0]); 
	std::cerr << buffer << std::endl;

	abort();
}

} // namespace


Hq4x::Hq4x(size_t textureWidth, size_t textureHeight) :
	m_textureWidth(textureWidth),
	m_textureHeight(textureHeight)
{
	m_vertexShader = glCreateShader(GL_VERTEX_SHADER);
	m_fragShader = glCreateShader(GL_FRAGMENT_SHADER);

	if (!m_vertexShader || !m_fragShader) abortGLError();

	glShaderSource(m_vertexShader, 1, &s_VectorSource, NULL);
	glShaderSource(m_fragShader, 1, &s_FragmentSource, NULL);

	glCompileShader(m_vertexShader);
	if (glGetError() != GL_NO_ERROR) abortShaderError(m_vertexShader);
	glCompileShader(m_fragShader);
	if (glGetError() != GL_NO_ERROR) abortShaderError(m_fragShader);

	m_program = glCreateProgram();
	if (!m_program) abortGLError();

	glAttachShader(m_program, m_vertexShader);
	if (glGetError() != GL_NO_ERROR) abortProgramError(m_program);
	glAttachShader(m_program, m_fragShader);
	if (glGetError() != GL_NO_ERROR) abortProgramError(m_program);
	glLinkProgram(m_program);
	if (glGetError() != GL_NO_ERROR) abortProgramError(m_program);

	glGenTextures(1, &m_lookupTexture);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, m_lookupTexture);

	// Only nearest neighbour will work
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

	// Create the texture image
	glTexImage2D(
		GL_TEXTURE_2D,
		0,
		GL_RGBA,
		48,
		4096,
		0,
		GL_RGBA,
		GL_UNSIGNED_BYTE,
		getHq4xLookupTable()
		);
	glUseProgram(m_program);
	if (glGetError() != GL_NO_ERROR) abortProgramError(m_program);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, m_lookupTexture);
	GLint tex = glGetUniformLocation(m_program, "mmLookup");
	glUniform1i(tex, 1); // Bind Texture 1
}

void
Hq4x::prepareShader(GLuint texture)
{
	glUseProgram(m_program);
	if (glGetError() != GL_NO_ERROR) abortProgramError(m_program);

	glBindFragDataLocationEXT(m_program,0,"fragColor");

	GLint size(glGetUniformLocation(m_program, "mmTextureSize"));
	glUniform4f(size, m_textureWidth, m_textureHeight, 0, 0);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, texture);
	GLint tex(glGetUniformLocation(m_program, "mmTexture"));
	glUniform1i(tex, 0); // Bind Texture 0

/*
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, m_lookupTexture);
	tex = glGetUniformLocation(m_program, "mmLookup");
	glUniform1i(tex, 1); // Bind Texture 1
*/
}

//--------------------------------------------------
// Lookup Table Generation Below
//--------------------------------------------------

struct weight
{
	weight() : w1(0),w2(0),w3(0),w4(0),w5(0),w6(0),w7(0),w8(0),w9(0){}
	uint32_t w1; uint32_t w2; uint32_t w3;
	uint32_t w4; uint32_t w5; uint32_t w6;
	uint32_t w7; uint32_t w8; uint32_t w9;
};

inline void Interp1(uint32_t& c1, uint32_t& c2)
{
	c1 = 191;
	c2 = 64;
}

inline void Interp2(uint32_t& c1, uint32_t& c2, uint32_t& c3)
{
	c1 = 127;
	c2 = 64;
	c3 = 64;
}

inline void Interp3(uint32_t& c1, uint32_t& c2)
{
	c1 = 223;
	c2 = 32;
}

inline void Interp5(uint32_t& c1, uint32_t& c2)
{
	c1 = 127;
	c2 = 128;
}

inline void Interp6(uint32_t& c1, uint32_t& c2, uint32_t& c3)
{
	c1 = 159;
	c2 = 64;
	c3 = 32;
}

inline void Interp7(uint32_t& c1, uint32_t& c2, uint32_t& c3)
{
	c1 = 191;
	c2 = 32;
	c3 = 32;
}

inline void Interp8(uint32_t& c1, uint32_t& c2)
{
	c1 = 159;
	c2 = 96;
}



#define p1 weights[0][0]
#define p2 weights[0][1]
#define p3 weights[0][2]
#define p4 weights[0][3]
#define p5 weights[1][0]
#define p6 weights[1][1]
#define p7 weights[1][2]
#define p8 weights[1][3]
#define p9 weights[2][0]
#define p10 weights[2][1]
#define p11 weights[2][2]
#define p12 weights[2][3]
#define p13 weights[3][0]
#define p14 weights[3][1]
#define p15 weights[3][2]
#define p16 weights[3][3]

#define PIXEL00_0     p1.w5 = 255;
#define PIXEL00_11    Interp1(p1.w5, p1.w4);
#define PIXEL00_12    Interp1(p1.w5, p1.w2);;
#define PIXEL00_20    Interp2(p1.w5, p1.w2, p1.w4);
#define PIXEL00_50    Interp5(p1.w2, p1.w4);
#define PIXEL00_80    Interp8(p1.w5, p1.w1);
#define PIXEL00_81    Interp8(p1.w5, p1.w4);
#define PIXEL00_82    Interp8(p1.w5, p1.w2);

#define PIXEL01_0     p2.w5 = 255;
#define PIXEL01_10    Interp1(p2.w5, p2.w1);
#define PIXEL01_12    Interp1(p2.w5, p2.w2);
#define PIXEL01_14    Interp1(p2.w2, p2.w5);
#define PIXEL01_21    Interp2(p2.w2, p2.w5, p2.w4);
#define PIXEL01_31    Interp3(p2.w5, p2.w4);
#define PIXEL01_50    Interp5(p2.w2, p2.w5);
#define PIXEL01_60    Interp6(p2.w5, p2.w2, p2.w4);
#define PIXEL01_61    Interp6(p2.w5, p2.w2, p2.w1);
#define PIXEL01_82    Interp8(p2.w5, p2.w2);
#define PIXEL01_83    Interp8(p2.w2, p2.w4);

#define PIXEL02_0     p3.w5 = 255;
#define PIXEL02_10    Interp1(p3.w5, p3.w3);
#define PIXEL02_11    Interp1(p3.w5, p3.w2);
#define PIXEL02_13    Interp1(p3.w2, p3.w5);
#define PIXEL02_21    Interp2(p3.w2, p3.w5, p3.w6);
#define PIXEL02_32    Interp3(p3.w5, p3.w6);
#define PIXEL02_50    Interp5(p3.w2, p3.w5);
#define PIXEL02_60    Interp6(p3.w5, p3.w2, p3.w6);
#define PIXEL02_61    Interp6(p3.w5, p3.w2, p3.w3);
#define PIXEL02_81    Interp8(p3.w5, p3.w2);
#define PIXEL02_83    Interp8(p3.w2, p3.w6);

#define PIXEL03_0     p4.w5 = 255;
#define PIXEL03_11    Interp1(p4.w5, p4.w2);
#define PIXEL03_12    Interp1(p4.w5, p4.w6);
#define PIXEL03_20    Interp2(p4.w5, p4.w2, p4.w6);
#define PIXEL03_50    Interp5(p4.w2, p4.w6);
#define PIXEL03_80    Interp8(p4.w5, p4.w3);
#define PIXEL03_81    Interp8(p4.w5, p4.w2);
#define PIXEL03_82    Interp8(p4.w5, p4.w6);

#define PIXEL10_0     p5.w5 = 255;
#define PIXEL10_10    Interp1(p5.w5, p5.w1);
#define PIXEL10_11    Interp1(p5.w5, p5.w4);
#define PIXEL10_13    Interp1(p5.w4, p5.w5);
#define PIXEL10_21    Interp2(p5.w4, p5.w5, p5.w2);
#define PIXEL10_32    Interp3(p5.w5, p5.w2);
#define PIXEL10_50    Interp5(p5.w4, p5.w5);
#define PIXEL10_60    Interp6(p5.w5, p5.w4, p5.w2);
#define PIXEL10_61    Interp6(p5.w5, p5.w4, p5.w1);
#define PIXEL10_81    Interp8(p5.w5, p5.w4);
#define PIXEL10_83    Interp8(p5.w4, p5.w2);

#define PIXEL11_0     p6.w5 = 255;
#define PIXEL11_30    Interp3(p6.w5, p6.w1);
#define PIXEL11_31    Interp3(p6.w5, p6.w4);
#define PIXEL11_32    Interp3(p6.w5, p6.w2);
#define PIXEL11_70    Interp7(p6.w5, p6.w4, p6.w2);

#define PIXEL12_0     p7.w5 = 255;
#define PIXEL12_30    Interp3(p7.w5, p7.w3);
#define PIXEL12_31    Interp3(p7.w5, p7.w2);
#define PIXEL12_32    Interp3(p7.w5, p7.w6);
#define PIXEL12_70    Interp7(p7.w5, p7.w6, p7.w2);

#define PIXEL13_0     p8.w5 = 255;
#define PIXEL13_10    Interp1(p8.w5, p8.w3);
#define PIXEL13_12    Interp1(p8.w5, p8.w6);
#define PIXEL13_14    Interp1(p8.w6, p8.w5);
#define PIXEL13_21    Interp2(p8.w6, p8.w5, p8.w2);
#define PIXEL13_31    Interp3(p8.w5, p8.w2);
#define PIXEL13_50    Interp5(p8.w6, p8.w5);
#define PIXEL13_60    Interp6(p8.w5, p8.w6, p8.w2);
#define PIXEL13_61    Interp6(p8.w5, p8.w6, p8.w3);
#define PIXEL13_82    Interp8(p8.w5, p8.w6);
#define PIXEL13_83    Interp8(p8.w6, p8.w2);

#define PIXEL20_0     p9.w5 = 255;
#define PIXEL20_10    Interp1(p9.w5, p9.w7);
#define PIXEL20_12    Interp1(p9.w5, p9.w4);
#define PIXEL20_14    Interp1(p9.w4, p9.w5);
#define PIXEL20_21    Interp2(p9.w4, p9.w5, p9.w8);
#define PIXEL20_31    Interp3(p9.w5, p9.w8);
#define PIXEL20_50    Interp5(p9.w4, p9.w5);
#define PIXEL20_60    Interp6(p9.w5, p9.w4, p9.w8);
#define PIXEL20_61    Interp6(p9.w5, p9.w4, p9.w7);
#define PIXEL20_82    Interp8(p9.w5, p9.w4);
#define PIXEL20_83    Interp8(p9.w4, p9.w8);

#define PIXEL21_0     p10.w5 = 255;
#define PIXEL21_30    Interp3(p10.w5, p10.w7);
#define PIXEL21_31    Interp3(p10.w5, p10.w8);
#define PIXEL21_32    Interp3(p10.w5, p10.w4);
#define PIXEL21_70    Interp7(p10.w5, p10.w4, p10.w8);

#define PIXEL22_0     p11.w5 = 255;
#define PIXEL22_30    Interp3(p11.w5, p11.w9);
#define PIXEL22_31    Interp3(p11.w5, p11.w6);
#define PIXEL22_32    Interp3(p11.w5, p11.w8);
#define PIXEL22_70    Interp7(p11.w5, p11.w6, p11.w8);

#define PIXEL23_0     p12.w5 = 255;
#define PIXEL23_10    Interp1(p12.w5, p12.w9);
#define PIXEL23_11    Interp1(p12.w5, p12.w6);
#define PIXEL23_13    Interp1(p12.w6, p12.w5);
#define PIXEL23_21    Interp2(p12.w6, p12.w5, p12.w8);
#define PIXEL23_32    Interp3(p12.w5, p12.w8);
#define PIXEL23_50    Interp5(p12.w6, p12.w5);
#define PIXEL23_60    Interp6(p12.w5, p12.w6, p12.w8);
#define PIXEL23_61    Interp6(p12.w5, p12.w6, p12.w9);
#define PIXEL23_81    Interp8(p12.w5, p12.w6);
#define PIXEL23_83    Interp8(p12.w6, p12.w8);

#define PIXEL30_0     p13.w5 = 255;
#define PIXEL30_11    Interp1(p13.w5, p13.w8);
#define PIXEL30_12    Interp1(p13.w5, p13.w4);
#define PIXEL30_20    Interp2(p13.w5, p13.w8, p13.w4);
#define PIXEL30_50    Interp5(p13.w8, p13.w4);
#define PIXEL30_80    Interp8(p13.w5, p13.w7);
#define PIXEL30_81    Interp8(p13.w5, p13.w8);
#define PIXEL30_82    Interp8(p13.w5, p13.w4);

#define PIXEL31_0     p14.w5 = 255;
#define PIXEL31_10    Interp1(p14.w5, p14.w7);
#define PIXEL31_11    Interp1(p14.w5, p14.w8);
#define PIXEL31_13    Interp1(p14.w8, p14.w5);
#define PIXEL31_21    Interp2(p14.w8, p14.w5, p14.w4);
#define PIXEL31_32    Interp3(p14.w5, p14.w4);
#define PIXEL31_50    Interp5(p14.w8, p14.w5);
#define PIXEL31_60    Interp6(p14.w5, p14.w8, p14.w4);
#define PIXEL31_61    Interp6(p14.w5, p14.w8, p14.w7);
#define PIXEL31_81    Interp8(p14.w5, p14.w8);
#define PIXEL31_83    Interp8(p14.w8, p14.w4);

#define PIXEL32_0     p15.w5 = 255;
#define PIXEL32_10    Interp1(p15.w5, p15.w9);
#define PIXEL32_12    Interp1(p15.w5, p15.w8);
#define PIXEL32_14    Interp1(p15.w8, p15.w5);
#define PIXEL32_21    Interp2(p15.w8, p15.w5, p15.w6);
#define PIXEL32_31    Interp3(p15.w5, p15.w6);
#define PIXEL32_50    Interp5(p15.w8, p15.w5);
#define PIXEL32_60    Interp6(p15.w5, p15.w8, p15.w6);
#define PIXEL32_61    Interp6(p15.w5, p15.w8, p15.w9);
#define PIXEL32_82    Interp8(p15.w5, p15.w8);
#define PIXEL32_83    Interp8(p15.w8, p15.w6);

#define PIXEL33_0     p16.w5 = 255;
#define PIXEL33_11    Interp1(p16.w5, p16.w6);
#define PIXEL33_12    Interp1(p16.w5, p16.w8);
#define PIXEL33_20    Interp2(p16.w5, p16.w8, p16.w6);
#define PIXEL33_50    Interp5(p16.w8, p16.w6);
#define PIXEL33_80    Interp8(p16.w5, p16.w9);
#define PIXEL33_81    Interp8(p16.w5, p16.w6);
#define PIXEL33_82    Interp8(p16.w5, p16.w8);



uint32_t*
Hq4x::getHq4xLookupTable() const
{
	static uint32_t* s_table(0);

	if (s_table)
	{
		return s_table;
	}
	else
	{
		s_table = new uint32_t[4096*48];

		for (int pattern = 0; pattern < 256; ++pattern)
		{
			for (int diff = 0; diff < 16; ++diff)
			{
				weight weights[4][4];

				switch (pattern)
				{
					case 0:
					case 1:
					case 4:
					case 32:
					case 128:
					case 5:
					case 132:
					case 160:
					case 33:
					case 129:
					case 36:
					case 133:
					case 164:
					case 161:
					case 37:
					case 165:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 2:
					case 34:
					case 130:
					case 162:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 16:
					case 17:
					case 48:
					case 49:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 64:
					case 65:
					case 68:
					case 69:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 8:
					case 12:
					case 136:
					case 140:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 3:
					case 35:
					case 131:
					case 163:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 6:
					case 38:
					case 134:
					case 166:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 20:
					case 21:
					case 52:
					case 53:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 144:
					case 145:
					case 176:
					case 177:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 192:
					case 193:
					case 196:
					case 197:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 96:
					case 97:
					case 100:
					case 101:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 40:
					case 44:
					case 168:
					case 172:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 9:
					case 13:
					case 137:
					case 141:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 18:
					case 50:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL12_0;
								PIXEL13_50;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 80:
					case 81:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_61;
							PIXEL21_30;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 72:
					case 76:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_70;
							PIXEL13_60;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_50;
								PIXEL21_0;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 10:
					case 138:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
								PIXEL11_0;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 66:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 24:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 7:
					case 39:
					case 135:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 148:
					case 149:
					case 180:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 224:
					case 228:
					case 225:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 41:
					case 169:
					case 45:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 22:
					case 54:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 208:
					case 209:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 104:
					case 108:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_70;
							PIXEL13_60;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 11:
					case 139:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 19:
					case 51:
						{
							if (diff & Diff_2_6)
							{
								PIXEL00_81;
								PIXEL01_31;
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL00_12;
								PIXEL01_14;
								PIXEL02_83;
								PIXEL03_50;
								PIXEL12_70;
								PIXEL13_21;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 146:
					case 178:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
								PIXEL23_32;
								PIXEL33_82;
							}
							else
							{
								PIXEL02_21;
								PIXEL03_50;
								PIXEL12_70;
								PIXEL13_83;
								PIXEL23_13;
								PIXEL33_11;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_32;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_82;
							break;
						}
					case 84:
					case 85:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_81;
							if (diff & Diff_6_8)
							{
								PIXEL03_81;
								PIXEL13_31;
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL03_12;
								PIXEL13_14;
								PIXEL22_70;
								PIXEL23_83;
								PIXEL32_21;
								PIXEL33_50;
							}
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_31;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 112:
					case 113:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_82;
							PIXEL21_32;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL30_82;
								PIXEL31_32;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_70;
								PIXEL23_21;
								PIXEL30_11;
								PIXEL31_13;
								PIXEL32_83;
								PIXEL33_50;
							}
							break;
						}
					case 200:
					case 204:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_70;
							PIXEL13_60;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
								PIXEL32_31;
								PIXEL33_81;
							}
							else
							{
								PIXEL20_21;
								PIXEL21_70;
								PIXEL30_50;
								PIXEL31_83;
								PIXEL32_14;
								PIXEL33_12;
							}
							PIXEL22_31;
							PIXEL23_81;
							break;
						}
					case 73:
					case 77:
						{
							if (diff & Diff_8_4)
							{
								PIXEL00_82;
								PIXEL10_32;
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL00_11;
								PIXEL10_13;
								PIXEL20_83;
								PIXEL21_70;
								PIXEL30_50;
								PIXEL31_21;
							}
							PIXEL01_82;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL11_32;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 42:
					case 170:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
								PIXEL20_31;
								PIXEL30_81;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_21;
								PIXEL10_83;
								PIXEL11_70;
								PIXEL20_14;
								PIXEL30_12;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL21_31;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL31_81;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 14:
					case 142:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL02_32;
								PIXEL03_82;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_83;
								PIXEL02_13;
								PIXEL03_11;
								PIXEL10_21;
								PIXEL11_70;
							}
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 67:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 70:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 28:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 152:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 194:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 98:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 56:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 25:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 26:
					case 31:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL11_0;
							PIXEL12_0;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 82:
					case 214:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 88:
					case 248:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							break;
						}
					case 74:
					case 107:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_61;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 27:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 86:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 216:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 106:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_61;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 30:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 210:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 120:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 75:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 29:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 198:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 184:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 99:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 57:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 71:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 156:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 226:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 60:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 195:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 102:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 153:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 58:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 83:
						{
							PIXEL00_81;
							PIXEL01_31;
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL20_61;
							PIXEL21_30;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 92:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 202:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL12_30;
							PIXEL13_61;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							PIXEL22_31;
							PIXEL23_81;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 78:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							PIXEL02_32;
							PIXEL03_82;
							PIXEL12_32;
							PIXEL13_82;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 154:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 114:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL20_82;
							PIXEL21_32;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							PIXEL30_82;
							PIXEL31_32;
							break;
						}
					case 89:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 90:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 55:
					case 23:
						{
							if (diff & Diff_2_6)
							{
								PIXEL00_81;
								PIXEL01_31;
								PIXEL02_0;
								PIXEL03_0;
								PIXEL12_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL00_12;
								PIXEL01_14;
								PIXEL02_83;
								PIXEL03_50;
								PIXEL12_70;
								PIXEL13_21;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 182:
					case 150:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL12_0;
								PIXEL13_0;
								PIXEL23_32;
								PIXEL33_82;
							}
							else
							{
								PIXEL02_21;
								PIXEL03_50;
								PIXEL12_70;
								PIXEL13_83;
								PIXEL23_13;
								PIXEL33_11;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_32;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_82;
							break;
						}
					case 213:
					case 212:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_81;
							if (diff & Diff_6_8)
							{
								PIXEL03_81;
								PIXEL13_31;
								PIXEL22_0;
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL03_12;
								PIXEL13_14;
								PIXEL22_70;
								PIXEL23_83;
								PIXEL32_21;
								PIXEL33_50;
							}
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_31;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 241:
					case 240:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_82;
							PIXEL21_32;
							if (diff & Diff_6_8)
							{
								PIXEL22_0;
								PIXEL23_0;
								PIXEL30_82;
								PIXEL31_32;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL22_70;
								PIXEL23_21;
								PIXEL30_11;
								PIXEL31_13;
								PIXEL32_83;
								PIXEL33_50;
							}
							break;
						}
					case 236:
					case 232:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_70;
							PIXEL13_60;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL21_0;
								PIXEL30_0;
								PIXEL31_0;
								PIXEL32_31;
								PIXEL33_81;
							}
							else
							{
								PIXEL20_21;
								PIXEL21_70;
								PIXEL30_50;
								PIXEL31_83;
								PIXEL32_14;
								PIXEL33_12;
							}
							PIXEL22_31;
							PIXEL23_81;
							break;
						}
					case 109:
					case 105:
						{
							if (diff & Diff_8_4)
							{
								PIXEL00_82;
								PIXEL10_32;
								PIXEL20_0;
								PIXEL21_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL00_11;
								PIXEL10_13;
								PIXEL20_83;
								PIXEL21_70;
								PIXEL30_50;
								PIXEL31_21;
							}
							PIXEL01_82;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL11_32;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 171:
					case 43:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
								PIXEL11_0;
								PIXEL20_31;
								PIXEL30_81;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_21;
								PIXEL10_83;
								PIXEL11_70;
								PIXEL20_14;
								PIXEL30_12;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL21_31;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL31_81;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 143:
					case 15:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL02_32;
								PIXEL03_82;
								PIXEL10_0;
								PIXEL11_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_83;
								PIXEL02_13;
								PIXEL03_11;
								PIXEL10_21;
								PIXEL11_70;
							}
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 124:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 203:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 62:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 211:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 118:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 217:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 110:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_32;
							PIXEL13_82;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 155:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 188:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 185:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 61:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 157:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 103:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 227:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 230:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 199:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 220:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							break;
						}
					case 158:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL12_0;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 234:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL12_30;
							PIXEL13_61;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 242:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_82;
							PIXEL31_32;
							break;
						}
					case 59:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL11_0;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 121:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 87:
						{
							PIXEL00_81;
							PIXEL01_31;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_0;
							PIXEL20_61;
							PIXEL21_30;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 79:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_32;
							PIXEL03_82;
							PIXEL11_0;
							PIXEL12_32;
							PIXEL13_82;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 122:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 94:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL12_0;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 218:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							break;
						}
					case 91:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL11_0;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 229:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 167:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 173:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 181:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 186:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 115:
						{
							PIXEL00_81;
							PIXEL01_31;
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL20_82;
							PIXEL21_32;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							PIXEL30_82;
							PIXEL31_32;
							break;
						}
					case 93:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL13_31;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							break;
						}
					case 206:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							PIXEL02_32;
							PIXEL03_82;
							PIXEL12_32;
							PIXEL13_82;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							PIXEL22_31;
							PIXEL23_81;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 205:
					case 201:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_70;
							PIXEL13_60;
							if (diff & Diff_8_4)
							{
								PIXEL20_10;
								PIXEL21_30;
								PIXEL30_80;
								PIXEL31_10;
							}
							else
							{
								PIXEL20_12;
								PIXEL21_0;
								PIXEL30_20;
								PIXEL31_11;
							}
							PIXEL22_31;
							PIXEL23_81;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 174:
					case 46:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_80;
								PIXEL01_10;
								PIXEL10_10;
								PIXEL11_30;
							}
							else
							{
								PIXEL00_20;
								PIXEL01_12;
								PIXEL10_11;
								PIXEL11_0;
							}
							PIXEL02_32;
							PIXEL03_82;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 179:
					case 147:
						{
							PIXEL00_81;
							PIXEL01_31;
							if (diff & Diff_2_6)
							{
								PIXEL02_10;
								PIXEL03_80;
								PIXEL12_30;
								PIXEL13_10;
							}
							else
							{
								PIXEL02_11;
								PIXEL03_20;
								PIXEL12_0;
								PIXEL13_12;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 117:
					case 116:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_82;
							PIXEL21_32;
							if (diff & Diff_6_8)
							{
								PIXEL22_30;
								PIXEL23_10;
								PIXEL32_10;
								PIXEL33_80;
							}
							else
							{
								PIXEL22_0;
								PIXEL23_11;
								PIXEL32_12;
								PIXEL33_20;
							}
							PIXEL30_82;
							PIXEL31_32;
							break;
						}
					case 189:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 231:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 126:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_0;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 219:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 125:
						{
							if (diff & Diff_8_4)
							{
								PIXEL00_82;
								PIXEL10_32;
								PIXEL20_0;
								PIXEL21_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL00_11;
								PIXEL10_13;
								PIXEL20_83;
								PIXEL21_70;
								PIXEL30_50;
								PIXEL31_21;
							}
							PIXEL01_82;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 221:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_81;
							if (diff & Diff_6_8)
							{
								PIXEL03_81;
								PIXEL13_31;
								PIXEL22_0;
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL03_12;
								PIXEL13_14;
								PIXEL22_70;
								PIXEL23_83;
								PIXEL32_21;
								PIXEL33_50;
							}
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 207:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL02_32;
								PIXEL03_82;
								PIXEL10_0;
								PIXEL11_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_83;
								PIXEL02_13;
								PIXEL03_11;
								PIXEL10_21;
								PIXEL11_70;
							}
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_31;
							PIXEL23_81;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 238:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_32;
							PIXEL13_82;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL21_0;
								PIXEL30_0;
								PIXEL31_0;
								PIXEL32_31;
								PIXEL33_81;
							}
							else
							{
								PIXEL20_21;
								PIXEL21_70;
								PIXEL30_50;
								PIXEL31_83;
								PIXEL32_14;
								PIXEL33_12;
							}
							PIXEL22_31;
							PIXEL23_81;
							break;
						}
					case 190:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL12_0;
								PIXEL13_0;
								PIXEL23_32;
								PIXEL33_82;
							}
							else
							{
								PIXEL02_21;
								PIXEL03_50;
								PIXEL12_70;
								PIXEL13_83;
								PIXEL23_13;
								PIXEL33_11;
							}
							PIXEL10_10;
							PIXEL11_30;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_82;
							break;
						}
					case 187:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
								PIXEL11_0;
								PIXEL20_31;
								PIXEL30_81;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_21;
								PIXEL10_83;
								PIXEL11_70;
								PIXEL20_14;
								PIXEL30_12;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL31_81;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 243:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_82;
							PIXEL21_32;
							if (diff & Diff_6_8)
							{
								PIXEL22_0;
								PIXEL23_0;
								PIXEL30_82;
								PIXEL31_32;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL22_70;
								PIXEL23_21;
								PIXEL30_11;
								PIXEL31_13;
								PIXEL32_83;
								PIXEL33_50;
							}
							break;
						}
					case 119:
						{
							if (diff & Diff_2_6)
							{
								PIXEL00_81;
								PIXEL01_31;
								PIXEL02_0;
								PIXEL03_0;
								PIXEL12_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL00_12;
								PIXEL01_14;
								PIXEL02_83;
								PIXEL03_50;
								PIXEL12_70;
								PIXEL13_21;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 237:
					case 233:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_60;
							PIXEL03_20;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_70;
							PIXEL13_60;
							PIXEL20_0;
							PIXEL21_0;
							PIXEL22_31;
							PIXEL23_81;
							if (diff & Diff_8_4)
							{
								PIXEL30_0;
							}
							else
							{
								PIXEL30_20;
							}
							PIXEL31_0;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 175:
					case 47:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
							}
							else
							{
								PIXEL00_20;
							}
							PIXEL01_0;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_0;
							PIXEL11_0;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_70;
							PIXEL23_60;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_60;
							PIXEL33_20;
							break;
						}
					case 183:
					case 151:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_0;
							if (diff & Diff_2_6)
							{
								PIXEL03_0;
							}
							else
							{
								PIXEL03_20;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_0;
							PIXEL13_0;
							PIXEL20_60;
							PIXEL21_70;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_20;
							PIXEL31_60;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 245:
					case 244:
						{
							PIXEL00_20;
							PIXEL01_60;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_60;
							PIXEL11_70;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_0;
							PIXEL23_0;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_0;
							if (diff & Diff_6_8)
							{
								PIXEL33_0;
							}
							else
							{
								PIXEL33_20;
							}
							break;
						}
					case 250:
						{
							PIXEL00_80;
							PIXEL01_10;
							PIXEL02_10;
							PIXEL03_80;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_30;
							PIXEL13_10;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							break;
						}
					case 123:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_10;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 95:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL11_0;
							PIXEL12_0;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_80;
							PIXEL31_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 222:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 252:
						{
							PIXEL00_80;
							PIXEL01_61;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_31;
							PIXEL13_31;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_0;
							PIXEL23_0;
							PIXEL32_0;
							if (diff & Diff_6_8)
							{
								PIXEL33_0;
							}
							else
							{
								PIXEL33_20;
							}
							break;
						}
					case 249:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_61;
							PIXEL03_80;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_0;
							PIXEL21_0;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							if (diff & Diff_8_4)
							{
								PIXEL30_0;
							}
							else
							{
								PIXEL30_20;
							}
							PIXEL31_0;
							break;
						}
					case 235:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_61;
							PIXEL20_0;
							PIXEL21_0;
							PIXEL22_31;
							PIXEL23_81;
							if (diff & Diff_8_4)
							{
								PIXEL30_0;
							}
							else
							{
								PIXEL30_20;
							}
							PIXEL31_0;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 111:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
							}
							else
							{
								PIXEL00_20;
							}
							PIXEL01_0;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_0;
							PIXEL11_0;
							PIXEL12_32;
							PIXEL13_82;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_61;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 63:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
							}
							else
							{
								PIXEL00_20;
							}
							PIXEL01_0;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_0;
							PIXEL11_0;
							PIXEL12_0;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_61;
							PIXEL33_80;
							break;
						}
					case 159:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_0;
							if (diff & Diff_2_6)
							{
								PIXEL03_0;
							}
							else
							{
								PIXEL03_20;
							}
							PIXEL11_0;
							PIXEL12_0;
							PIXEL13_0;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_80;
							PIXEL31_61;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 215:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_0;
							if (diff & Diff_2_6)
							{
								PIXEL03_0;
							}
							else
							{
								PIXEL03_20;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_0;
							PIXEL13_0;
							PIXEL20_61;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 246:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_61;
							PIXEL11_30;
							PIXEL12_0;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_0;
							PIXEL23_0;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_0;
							if (diff & Diff_6_8)
							{
								PIXEL33_0;
							}
							else
							{
								PIXEL33_20;
							}
							break;
						}
					case 254:
						{
							PIXEL00_80;
							PIXEL01_10;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_10;
							PIXEL11_30;
							PIXEL12_0;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_0;
							PIXEL23_0;
							PIXEL32_0;
							if (diff & Diff_6_8)
							{
								PIXEL33_0;
							}
							else
							{
								PIXEL33_20;
							}
							break;
						}
					case 253:
						{
							PIXEL00_82;
							PIXEL01_82;
							PIXEL02_81;
							PIXEL03_81;
							PIXEL10_32;
							PIXEL11_32;
							PIXEL12_31;
							PIXEL13_31;
							PIXEL20_0;
							PIXEL21_0;
							PIXEL22_0;
							PIXEL23_0;
							if (diff & Diff_8_4)
							{
								PIXEL30_0;
							}
							else
							{
								PIXEL30_20;
							}
							PIXEL31_0;
							PIXEL32_0;
							if (diff & Diff_6_8)
							{
								PIXEL33_0;
							}
							else
							{
								PIXEL33_20;
							}
							break;
						}
					case 251:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_10;
							PIXEL03_80;
							PIXEL11_0;
							PIXEL12_30;
							PIXEL13_10;
							PIXEL20_0;
							PIXEL21_0;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							if (diff & Diff_8_4)
							{
								PIXEL30_0;
							}
							else
							{
								PIXEL30_20;
							}
							PIXEL31_0;
							break;
						}
					case 239:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
							}
							else
							{
								PIXEL00_20;
							}
							PIXEL01_0;
							PIXEL02_32;
							PIXEL03_82;
							PIXEL10_0;
							PIXEL11_0;
							PIXEL12_32;
							PIXEL13_82;
							PIXEL20_0;
							PIXEL21_0;
							PIXEL22_31;
							PIXEL23_81;
							if (diff & Diff_8_4)
							{
								PIXEL30_0;
							}
							else
							{
								PIXEL30_20;
							}
							PIXEL31_0;
							PIXEL32_31;
							PIXEL33_81;
							break;
						}
					case 127:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
							}
							else
							{
								PIXEL00_20;
							}
							PIXEL01_0;
							if (diff & Diff_2_6)
							{
								PIXEL02_0;
								PIXEL03_0;
								PIXEL13_0;
							}
							else
							{
								PIXEL02_50;
								PIXEL03_50;
								PIXEL13_50;
							}
							PIXEL10_0;
							PIXEL11_0;
							PIXEL12_0;
							if (diff & Diff_8_4)
							{
								PIXEL20_0;
								PIXEL30_0;
								PIXEL31_0;
							}
							else
							{
								PIXEL20_50;
								PIXEL30_50;
								PIXEL31_50;
							}
							PIXEL21_0;
							PIXEL22_30;
							PIXEL23_10;
							PIXEL32_10;
							PIXEL33_80;
							break;
						}
					case 191:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
							}
							else
							{
								PIXEL00_20;
							}
							PIXEL01_0;
							PIXEL02_0;
							if (diff & Diff_2_6)
							{
								PIXEL03_0;
							}
							else
							{
								PIXEL03_20;
							}
							PIXEL10_0;
							PIXEL11_0;
							PIXEL12_0;
							PIXEL13_0;
							PIXEL20_31;
							PIXEL21_31;
							PIXEL22_32;
							PIXEL23_32;
							PIXEL30_81;
							PIXEL31_81;
							PIXEL32_82;
							PIXEL33_82;
							break;
						}
					case 223:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
								PIXEL01_0;
								PIXEL10_0;
							}
							else
							{
								PIXEL00_50;
								PIXEL01_50;
								PIXEL10_50;
							}
							PIXEL02_0;
							if (diff & Diff_2_6)
							{
								PIXEL03_0;
							}
							else
							{
								PIXEL03_20;
							}
							PIXEL11_0;
							PIXEL12_0;
							PIXEL13_0;
							PIXEL20_10;
							PIXEL21_30;
							PIXEL22_0;
							if (diff & Diff_6_8)
							{
								PIXEL23_0;
								PIXEL32_0;
								PIXEL33_0;
							}
							else
							{
								PIXEL23_50;
								PIXEL32_50;
								PIXEL33_50;
							}
							PIXEL30_80;
							PIXEL31_10;
							break;
						}
					case 247:
						{
							PIXEL00_81;
							PIXEL01_31;
							PIXEL02_0;
							if (diff & Diff_2_6)
							{
								PIXEL03_0;
							}
							else
							{
								PIXEL03_20;
							}
							PIXEL10_81;
							PIXEL11_31;
							PIXEL12_0;
							PIXEL13_0;
							PIXEL20_82;
							PIXEL21_32;
							PIXEL22_0;
							PIXEL23_0;
							PIXEL30_82;
							PIXEL31_32;
							PIXEL32_0;
							if (diff & Diff_6_8)
							{
								PIXEL33_0;
							}
							else
							{
								PIXEL33_20;
							}
							break;
						}
					case 255:
						{
							if (diff & Diff_4_2)
							{
								PIXEL00_0;
							}
							else
							{
								PIXEL00_20;
							}
							PIXEL01_0;
							PIXEL02_0;
							if (diff & Diff_2_6)
							{
								PIXEL03_0;
							}
							else
							{
								PIXEL03_20;
							}
							PIXEL10_0;
							PIXEL11_0;
							PIXEL12_0;
							PIXEL13_0;
							PIXEL20_0;
							PIXEL21_0;
							PIXEL22_0;
							PIXEL23_0;
							if (diff & Diff_8_4)
							{
								PIXEL30_0;
							}
							else
							{
								PIXEL30_20;
							}
							PIXEL31_0;
							PIXEL32_0;
							if (diff & Diff_6_8)
							{
								PIXEL33_0;
							}
							else
							{
								PIXEL33_20;
							}
							break;
						}
				}

				for (int pixely = 0; pixely < 4; ++pixely)
				{
					for (int pixelx = 0; pixelx < 4; ++pixelx)
					{
						weight w(weights[pixely][pixelx]);

						assert(w.w1+w.w2+w.w3+w.w4+w.w5+w.w6+w.w7+w.w8+w.w9 == 255);
						size_t index(
							((pattern << 4) | diff) * 48 + (pixely*4+pixelx)*3
							);


// TODO byteorder fixes here!
						s_table[index] =
							(w.w1 << 0) |
							(w.w2 << 8) |
							(w.w3 << 16);


						s_table[index + 1] =
							(w.w4 << 0) |
							(w.w5 << 8) |
							(w.w6 << 16);

						s_table[index + 2] =
							(w.w7 << 0) |
							(w.w8 << 8) |
							(w.w9 << 16);

//hmm, order appears to be ABGR
/*
						s_table[index] = 0x0090FF10;
						s_table[index+1] = 0x00;
						s_table[index+2] = 0x00;
*/
					}
				}
			}
		}
	}

	return s_table;
}

